using System;
using Autofac.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Configuration;
using Serilog;
using MediatR;
using IRaCIS.Core.Application.MediatR.Handlers;
using System.Threading.Tasks;
using MassTransit;
using MassTransit.NewIdProviders;
using System.IO;
using Newtonsoft.Json.Linq;
using System.Security.Cryptography;
using IRaCIS.Core.Infrastructure;
using System.Net.NetworkInformation;
using System.Linq;

namespace IRaCIS.Core.API
{
    public class Program
    {
        public readonly string environment;
        public static async Task Main(string[] args)
        {

            #region ݿдļ

            var currentDicectory = AppContext.BaseDirectory;

            var rootPath = (Directory.GetParent(currentDicectory.TrimEnd('\\'))).FullName;
            var configPath = Path.Combine(rootPath, "resources/config.json");

            var initConfigJson = File.ReadAllText(configPath);

            var innitJsonObj = JObject.Parse(initConfigJson);

            var dbServerName = innitJsonObj["dbServerName"].ToString();
            var dbLoginUser = innitJsonObj["dbLoginUser"].ToString();
            var dbPwd = innitJsonObj["dbPwd"].ToString();


            var appsettingsJson = File.ReadAllText("appsettings.CertificateApply.json");

            //  JSON ַ
            var jObject = JObject.Parse(appsettingsJson);

            // ȡ UpdateConfig ڵĽڵ
            var updateConfigNode = jObject["ConnectionStrings"];


            updateConfigNode["RemoteNew"] = $"Server={dbServerName};Database=IRaCIS;User ID={dbLoginUser};Password={dbPwd}11;TrustServerCertificate=true";

            #endregion

            try
            {
                //ļΪ׼ urlȡֵ(дݲļ˾ͲҪݻ)
                var config = new ConfigurationBuilder()
                            .AddEnvironmentVariables()
                            .Build();

                var enviromentName = config["ASPNETCORE_ENVIRONMENT"];

                if (string.IsNullOrWhiteSpace(enviromentName))
                {

                    var index = Array.IndexOf(args, "--env");
                    enviromentName = index > -1
                        ? args[index + 1]
                        : "Development";
                }

                //Dicom 
                //ImageManager.SetImplementation(WinFormsImageManager.Instance);

                var host = CreateHostBuilder(args)
                    .UseEnvironment(enviromentName) //д뻷
                    .ConfigureAppConfiguration((hostContext, config) =>
                    {

                        //Console.WriteLine(hostContext.HostingEnvironment.EnvironmentName);                
                        config.AddJsonFile("appsettings.json", false, true)
                                  .AddJsonFile($"appsettings.{enviromentName}.json", false, true);
                    })
                    .Build();



                NewId.SetProcessIdProvider(new CurrentProcessIdProvider());




                //// Serilog 
                SerilogExtension.AddSerilogSetup(enviromentName, host.Services);

                #region ֤

                if (!File.Exists($@"C:\ProgramData\.xingcang\config.json"))
                {
                    Console.WriteLine("ǰδע");
                    Log.Logger.Error("ǰδע");
                    Console.ReadLine();
                    return;
                }
                else
                {
                    var json = File.ReadAllText($@"C:\ProgramData\.xingcang\config.json");

                    JObject jsonObject = JObject.Parse(json);

                    var key = jsonObject["key"].ToString();

                    var value = jsonObject["value"].ToString();


                    var physicalAddressList = NetworkInterface.GetAllNetworkInterfaces().Select(t => t.GetPhysicalAddress().ToString());

                    //// жļĻǷǱ
                    //if (!physicalAddressList.Contains(key))
                    //{
                    //    Console.WriteLine("ͱӦ");
                    //    Log.Logger.Error("ͱӦ");
                    //    Console.ReadLine();

                    //    return;
                    //}


                    var secrete = MD5Helper.Md5($"{key}_XINGCANG");

                    if (value != secrete)
                    {
                        Console.WriteLine("ע벻ƥ");
                        Log.Logger.Error("ע벻ƥ");
                        Console.ReadLine();

                        return;

                    }

                }

                #endregion





                //Ŀ״̬  
                await InitCache(host);



                host.Run();

                Log.Logger.Warning($"ǰ{enviromentName}");

            }
            catch (Exception e)
            {

                Log.Logger.Error(e.InnerException is null ? e.Message + e.StackTrace : e.InnerException?.Message + e.InnerException?.StackTrace);
            }



            #region Nlog 
            //var logger = NLog.Web.NLogBuilder.ConfigureNLog("NLog.config").GetCurrentClassLogger();
            //try
            //{
            //    var ihostBuilder = CreateHostBuilder(args);

            //    ihostBuilder.UseEnvironment(environment);

            //    ihostBuilder.ConfigureAppConfiguration((hostContext, config) =>
            //    {

            //        //Console.WriteLine(hostContext.HostingEnvironment.EnvironmentName);                
            //        config.AddJsonFile("appsettings.json", false, true)
            //              .AddJsonFile($"appsettings.{environment}.json", false, true);
            //    });

            //    var host = ihostBuilder.Build();

            //    CacheTrialStatus(host);

            //    host.Run();

            //}
            //catch (Exception exception)
            //{
            //    //NLog: catch setup errors
            //    logger.Error(exception, "Stopped program because of exception");
            //}
            //finally
            //{
            //    // Ensure to flush and stop internal timers/threads before application-exit (Avoid segmentation fault on Linux)
            //    NLog.LogManager.Shutdown();
            //}

            #endregion



        }

        public static IHostBuilder CreateHostBuilder(string[] args) =>
            Host.CreateDefaultBuilder(args)
                .UseWindowsService()
                .ConfigureWebHostDefaults(webBuilder =>
                {
                    webBuilder.ConfigureKestrel((context, options) =>
                     {
                         //Ӧ÷KestrelΪ1GB // if don't set default value is: 30 MB
                         options.Limits.MaxRequestBodySize = long.MaxValue;
                         options.Limits.KeepAliveTimeout = TimeSpan.FromMinutes(30);
                         options.Limits.RequestHeadersTimeout = TimeSpan.FromMinutes(20);

                     });
                    //webBuilder.UseSerilog();//ʱserilog,΢ILogger
                    webBuilder.UseStartup<Startup>();
                }).UseSerilog()
                .UseServiceProviderFactory(new AutofacServiceProviderFactory());//ʹAutofac
                                                                                //ConfigureLogging(logging =>
                                                                                //{
                                                                                //    logging.ClearProviders();
                                                                                //    logging.SetMinimumLevel(Microsoft.Extensions.Logging.LogLevel.Trace);
                                                                                //})
                                                                                //.UseNLog()
                                                                                // NLog: Setup NLog for Dependency injection;




        private static async Task InitCache(IHost host)
        {
            var _mediator = host.Services.GetService(typeof(IMediator)) as IMediator;

            await _mediator.Send(new AnonymizeCacheRequest());

            await _mediator.Send(new TrialStateCacheRequest());
        }

    }
}
